#! /usr/bin/env python
"""From http://stackoverflow.com/a/12260597/400691"""
import sys

from colour_runner.django_runner import ColourRunnerMixin
from django.conf import settings

import dj_database_url


settings.configure(
    DATABASES={
        'default': dj_database_url.config(default='postgres://localhost/feincms_extensions'),
    },
    DEFAULT_FILE_STORAGE='inmemorystorage.InMemoryStorage',
    INSTALLED_APPS=(
        # Put contenttypes before auth to work around test issue.
        # See: https://code.djangoproject.com/ticket/10827#comment:12
        'django.contrib.sites',
        'django.contrib.contenttypes',
        'django.contrib.auth',
        'django.contrib.sessions',
        'django.contrib.admin',

        'feincms',
        'feincms.module.medialibrary',
        'feincms.module.page',

        'feincms_extensions',
        'feincms_extensions.tests',

        'mptt',
    ),
    PASSWORD_HASHERS = ('django.contrib.auth.hashers.MD5PasswordHasher',),
    ROOT_URLCONF='feincms_extensions.tests.urls',
    SITE_ID = 1,
    MIDDLEWARE_CLASSES = (),
    TEMPLATE_CONTEXT_PROCESSORS = (
        'feincms.context_processors.add_page_if_missing',
    ),
)

import django
if django.VERSION >= (1, 7):
    django.setup()

from django.test.runner import DiscoverRunner


class TestRunner(ColourRunnerMixin, DiscoverRunner):
    pass


test_runner = TestRunner(verbosity=1)
failures = test_runner.run_tests(['feincms_extensions'])
if failures:
    sys.exit(1)



from feincms.content.medialibrary.models import MediaFileContent
from feincms.content.richtext.models import RichTextContent
from feincms.content.section.models import SectionContent


class JsonRichTextContent(RichTextContent):
    class Meta(RichTextContent.Meta):
        abstract = True

    def json(self, **kwargs):
        """Return a json serializable dictionary containing the content."""
        return {'html': self.text}


def mediafile_data(mediafile):
    """Return json serializable data for the mediafile."""
    if mediafile is None:
        return None
    return {
        'url': mediafile.file.url,
        'type': mediafile.type,
        'created': mediafile.created,
        'copyright': mediafile.copyright,
        'file_size': mediafile.file_size,
    }


class JsonSectionContent(SectionContent):
    class Meta(SectionContent.Meta):
        abstract = True

    def json(self, **kwargs):
        """Return a json serializable dictionary containing the content."""
        return {
            'title': self.title,
            'html': self.richtext,
            'mediafile': mediafile_data(self.mediafile),
        }


class JsonMediaFileContent(MediaFileContent):
    class Meta(MediaFileContent.Meta):
        abstract = True

    def json(self, **kwargs):
        """Return a json serializable dictionary containing the content."""
        return mediafile_data(self.mediafile)



from django.db import models


class TestContent(models.Model):
    """A stub feincms content type"""
    class Meta:
        abstract = True

    def render(self, **kwargs):
        return ''

    def json(self, **kwargs):
        return ''



from django.views.generic import TemplateView

from .factories import PageFactory
from .utils import RenderedContentTestCase


class DummyView(TemplateView):
    template_name = 'feincms_extensions/navigation.html'


class DummyViewCustomTemplate(TemplateView):
    template_name = 'navigation_test.html'


class PageMixin:
    def setUp(self):
        self.page = PageFactory.create(in_navigation=True, override_url='/')


class TestNoPage(RenderedContentTestCase):
    view = DummyView

    def test_failing(self):
        """At least one page is required when using the Page module from FeinCMS."""
        with self.assertRaises(AttributeError):
            self.access_view_and_render_response()


class TestPageMenuNavigation(PageMixin, RenderedContentTestCase):
    view = DummyView

    def test_menu(self):
        response = self.access_view_and_render_response()

        template = """
        <ul id="top-level">
            <li class=" selected">
                <span><a href="{}">{}</a></span>
            </li>
        </ul>
        """
        expected = template.format(self.page.override_url, self.page.title)
        self.assertHTMLEqual(response, expected)


class TestPageMenuCustomNavigation(PageMixin, RenderedContentTestCase):
    view = DummyViewCustomTemplate

    def test_nav_id(self):
        response = self.access_view_and_render_response()

        template = """
        <ul id="new-id">
            <li class=" selected">
                <span><a href="{}">{}</a></span>
            </li>
        </ul>
        """
        expected = template.format(self.page.override_url, self.page.title)
        self.assertHTMLEqual(response, expected)

    def test_depth(self):
        child = PageFactory.create(
            in_navigation=True,
            override_url='/child-page/',
            parent=self.page,
        )

        response = self.access_view_and_render_response()

        template = """
        <ul id="new-id">
            <li class="selected has-children">
                <span><a href="{}">{}</a></span>
            </li>
            <ul>
                <li class=" ">
                    <span><a href="{}">{}</a></span>
                </li>
            </ul>
        </ul>
        """
        expected = template.format(
            self.page.override_url,
            self.page.title,
            child.override_url,
            child.title,
        )
        self.assertHTMLEqual(response, expected)

    def test_depth_not_rendered(self):
        child = PageFactory.create(
            in_navigation=True,
            override_url='/child-page/',
            parent=self.page,
        )

        grand_child = PageFactory.create(
            in_navigation=True,
            override_url='/grand-child-page/',
            parent=child,
        )

        response = self.access_view_and_render_response()
        self.assertNotIn(grand_child.title, response)



import unittest

import django
from django.core import checks
from django.test import TestCase, TransactionTestCase
from feincms.content.richtext.models import RichTextContent
from mock import patch, MagicMock

from .factories import DummyFactory
from .models import Dummy


skip_if_checks_unavailable = unittest.skipIf(
    django.VERSION < (1, 7),
    'Checks only available in django>=1.7',
)


class TestExtension(TestCase):
    def test_json_regions(self):
        request = MagicMock()

        dummy = DummyFactory.create()
        json_path = 'feincms_extensions.tests.content.TestContent.json'
        expected_dict = {'foo': 'bar'}
        with patch(json_path) as json:
            json.return_value = expected_dict
            self.assertEqual(
                dummy.render_json(request),
                {'body': [expected_dict]},
            )

        json.assert_called_once_with(request=request)


class TestExtensionChecks(TransactionTestCase):
    @skip_if_checks_unavailable
    def test_check_content_types_json(self):
        rich_text_content = Dummy.create_content_type(RichTextContent)

        message = (
            'Feincms content has no `json` method, but the `render_json` ' +
            'extension is active for model `{}`.'
        ).format(Dummy)
        expected = [
            checks.Error(
                message,
                obj=rich_text_content,
                id='feincms_extensions.E001',
            ),
        ]
        self.assertEqual(Dummy.check(), expected)



from django.core import checks
from feincms import extensions


class Extension(extensions.Extension):
    def handle_model(self):
        cls = self.model

        def render_json(self, request):
            """Render the feincms regions into a dictionary."""

            def region_data(region):
                content_list = getattr(self.content, region.key)
                return [content.json(request=request) for content in content_list]

            regions = self.template.regions
            return {region.key: region_data(region) for region in regions}

        cls.add_to_class('render_json', render_json)

        @classmethod
        def check(cls, **kwargs):
            errors = super(self.model, cls).check(**kwargs)
            errors.extend(cls._check_json_method())
            return errors

        @classmethod
        def _check_json_method(cls, **kwargs):
            """Check all registered content types have a `.json` method."""
            message = (
                'Feincms content has no `json` method, but the ' +
                '`render_json` extension is active for model `{}`.'
            ).format(cls)
            for content_type in cls._feincms_content_types:
                if not hasattr(content_type, 'json'):
                    yield checks.Error(
                        message,
                        obj=content_type,
                        id='feincms_extensions.E001',
                    )

        cls.add_to_class('check', check)
        cls.add_to_class('_check_json_method', _check_json_method)



from django.contrib.auth import get_user_model
import factory
from feincms.module.medialibrary.models import MediaFile
from feincms.module.page.models import Page

from . import content, models


User = get_user_model()


class DummyFactory(factory.DjangoModelFactory):
    FACTORY_FOR = models.Dummy

    @factory.post_generation
    def content(self, create, extracted, **kwargs):
        if not create:
            return
        Content = models.Dummy.content_type_for(content.TestContent)
        Content.objects.create(parent=self, region='body')


class PageFactory(factory.DjangoModelFactory):
    FACTORY_FOR = Page

    title = factory.Sequence('Page {}'.format)


class UserFactory(factory.DjangoModelFactory):
    FACTORY_FOR = User

    email = factory.Sequence('{}@example.com'.format)


class MediaFileFactory(factory.DjangoModelFactory):
    file = factory.django.FileField(
        from_path='feincms_extensions/tests/images/image.png',
    )

    @factory.post_generation
    def file_size(self, create, extracted, **kwargs):
        """Duplicate MediaFile.save to set self.file_size in build."""
        self.file_size = self.file.size

    class Meta:
        model = MediaFile



from django.db import models
from django.utils.translation import ugettext_lazy as _
from feincms import extensions
from feincms.module.page.models import Page


class Extension(extensions.Extension):
    """
    Add a prepared date value to the FeinCMS `page.Page` model.
    If the page has no value set then the value is taken from nearest ancestor
    page (based on `_cached_url` path) that has a value.
    """

    def handle_model(self):
        self.model.add_to_class(
            '_prepared_date',
            models.TextField(
                'Date of Preparation',
                default='',
                blank=True,
                help_text=_('If not set the value from the closest parent page will be used.'),
            ),
        )

        def getter(obj):
            if obj._prepared_date:
                return obj._prepared_date

            # Find the best matching parent page (based on url) that has a _prepared_date
            tokens = obj.get_absolute_url().strip('/').split('/')
            paths = ['/'] + ['/%s/' % '/'.join(tokens[:i]) for i in range(1, len(tokens) + 1)]
            try:
                return Page.objects.apply_active_filters(
                    Page.objects.exclude(
                        _prepared_date=''
                    ).filter(
                        _cached_url__in=paths
                    ).extra(
                        select={'_url_length': 'LENGTH(_cached_url)'}
                    ).order_by(
                        '-_url_length'
                    )
                )[0]._prepared_date
            except IndexError:
                return ''

        def setter(obj, value):
            obj._prepared_date = value

        self.model.prepared_date = property(getter, setter)

    def handle_modeladmin(self, modeladmin):
        modeladmin.add_extension_options(_('Date of Preparation'), {
            'fields': ('_prepared_date',),
            'classes': ('collapse',),
        })



from feincms.models import create_base_model
from feincms.module.page.models import Page

from .content import TestContent
from .. import content_types


TYPE_CHOICES = (
    ('block', 'Block'),
)


class Dummy(create_base_model()):
    """A fake class for holding content"""


Dummy.register_regions(('body', 'Main'))
Dummy.create_content_type(TestContent)
Dummy.create_content_type(content_types.JsonRichTextContent)
Dummy.create_content_type(
    content_types.JsonMediaFileContent,
    TYPE_CHOICES=TYPE_CHOICES,
)
Dummy.create_content_type(
    content_types.JsonSectionContent,
    TYPE_CHOICES=TYPE_CHOICES,
)
Dummy.register_extensions(
    'feincms_extensions.render_regions',
    'feincms_extensions.render_json',
)

Page.register_templates({
    'key': 'key',
    'title': 'Title',
    'path': 'base.html',
    'regions': (
        ('body', 'Main'),
    ),
})
Page.register_extensions(
    'feincms_extensions.prepared_date',
)



from incuna_test_utils.testcases.integration import BaseIntegrationTestCase

from .factories import UserFactory


class RenderedContentTestCase(BaseIntegrationTestCase):
    user_factory = UserFactory



from django.contrib.admin.sites import AdminSite
from django.test import TestCase
from feincms.module.page.admin import PageAdmin
from feincms.module.page.models import Page
import mock

from .factories import PageFactory


class TestExtension(TestCase):
    def test_prepared_date_empty(self):
        page = PageFactory.create()

        self.assertEqual(page.prepared_date, '')

    def test_set_prepared_date(self):
        prepared_date = 'Sample data'
        page = PageFactory.create()
        page.prepared_date = prepared_date
        self.assertEqual(page.prepared_date, prepared_date)

    def test_prepared_date_parent(self):
        prepared_date = 'Sample data'
        parent = PageFactory.create(prepared_date=prepared_date, slug='a')
        page = PageFactory.create(parent=parent, slug='b')

        self.assertEqual(page.prepared_date, prepared_date)

    def test_prepared_date_override_url(self):
        prepared_date = 'Sample data'
        PageFactory.create(prepared_date=prepared_date, override_url='/')
        page = PageFactory.create(slug='page')

        self.assertEqual(page.prepared_date, prepared_date)

    def test_handle_modeladmin(self):
        page_admin = PageAdmin(Page, AdminSite())
        form = page_admin.get_form(mock.Mock())()
        self.assertIn('_prepared_date', form._meta.fields)



from feincms import extensions


class Extension(extensions.Extension):
    def handle_model(self):
        cls = self.model

        def rendered_regions(self, request):
            """Render the feincms regions into a dictionary."""
            regions = {}
            for region in self.template.regions:
                content_list = getattr(self.content, region.key)
                content = [c.render(request=request) for c in content_list]
                regions[region.key] = ' '.join(content)
            return regions
        cls.add_to_class('rendered_regions', rendered_regions)



from setuptools import setup, find_packages


version = '0.4.0'


install_requires = (
    'FeinCMS>=1.9,<1.11',
)

setup(
    name='feincms-extensions',
    packages=find_packages(),
    include_package_data=True,
    version=version,
    description='',
    long_description='',
    author='Incuna',
    author_email='admin@incuna.com',
    url='https://github.com/incuna/feincms-extensions/',
    install_requires=install_requires,
    zip_safe=False,
)



import datetime

from django.test import TestCase

from . import factories
from .models import Dummy
from .. import content_types


class TestJsonRichTextContent(TestCase):
    model = Dummy.content_type_for(content_types.JsonRichTextContent)

    def test_json(self):
        """A JsonRichTextContent can be rendered to json."""
        text = 'Rich Text'
        content = self.model(region='body', text=text)
        self.assertEqual(content.json(), {'html': text})


class TestJsonSectionContent(TestCase):
    model = Dummy.content_type_for(content_types.JsonSectionContent)

    def test_json(self):
        """A JsonSectionContent can be rendered to json."""
        title = 'Section 1'
        richtext = 'Rich Text'
        image_type = 'image'
        copyright = 'Incuna'
        created = datetime.datetime(year=2015, month=3, day=1)

        image = factories.MediaFileFactory.build(
            type=image_type,
            copyright=copyright,
            created=created,
        )
        content = self.model(
            region='body',
            title=title,
            richtext=richtext,
            mediafile=image,
        )

        expected = {
            'title': title,
            'html': richtext,
            'mediafile': {
                'url': image.file.url,
                'type': image_type,
                'created': created,
                'copyright': copyright,
                'file_size': image.file.size,
            },
        }
        self.assertEqual(content.json(), expected)

    def test_json_no_mediafile(self):
        """A JsonSectionContent can be rendered to json."""
        title = 'Section 1'
        richtext = 'Rich Text'

        content = self.model(
            region='body',
            title=title,
            richtext=richtext,
            mediafile=None,
        )

        expected = {
            'title': title,
            'html': richtext,
            'mediafile': None,
        }
        self.assertEqual(content.json(), expected)


class TestJsonMediaFileContent(TestCase):
    model = Dummy.content_type_for(content_types.JsonMediaFileContent)

    def test_json(self):
        """A JsonMediaFileContent can be rendered to json."""
        image_type = 'image'
        copyright = 'Incuna'
        created = datetime.datetime(year=2015, month=3, day=1)

        image = factories.MediaFileFactory.build(
            type=image_type,
            copyright=copyright,
            created=created,
        )
        content = self.model(region='body', mediafile=image)

        expected = {
            'url': image.file.url,
            'type': image_type,
            'created': created,
            'copyright': copyright,
            'file_size': image.file.size,
        }
        self.assertEqual(content.json(), expected)



from django.test import TestCase
from mock import patch, MagicMock

from .factories import DummyFactory


class TestExtension(TestCase):
    def test_rendered_regions(self):
        request = MagicMock()

        dummy = DummyFactory.create()
        render_path = 'feincms_extensions.tests.content.TestContent.render'
        with patch(render_path) as render:
            render.return_value = ''
            dummy.rendered_regions(request)

        render.assert_called_once_with(request=request)
